// /////////////////////////////////////////////////////
// Effektiv C++
// Tip 5: Sichere Typkonvertierungen
//
// Code: Stephan Brumme, 15.Mai 2002
//
// Zweck: - Demonstration aller Cast-Operatoren



// vorher RTTI fr dynamic_cast aktivieren !!!
// (Visual C++ 6: Compiler-Option /GR+)


// fr I/O
#include <iostream>
using namespace std;



// Basisklasse
class CBase
{
public:
	void ShowClass()
	{
		cout << "CBase" << endl;
	}

	virtual ~CBase() {}
};


// abgeleitete Klasse
class CDerived : public CBase
{
public:
	void ShowClass()
	{
		cout << "CDerived" << endl;
	}
	int m_nNumber;
};



// Hauptprogramm
void main()
{
	// ein Objekt der abgeleiteten Klasse
	CDerived *pDerived = new CDerived;
	// ein Zeiger auf die Basisklasse
	CBase    *pBase = pDerived;


	// zeige korrekte Klasse von pDerived
	pDerived->ShowClass();
	// pBase zeigt auf pDerived, Klasseninformation aber verloren
	pBase->ShowClass();

	///////////////////////////////////////////////////
	// dynamic_cast:

	// Fehler: Downcast muss explizit erfolgen
	// pDerived = pBase;
	// okay mit dynamic_cast:
	pDerived = dynamic_cast<CDerived*>(pBase);
	if (pDerived != NULL)
		pDerived->ShowClass();
	else
		cout << "Cast gescheitert !" << endl;
	// Rckgabewert ist NULL, wenn Cast nicht erfolgreich
	// pZahl als int* kann nicht in CDerived* konvertiert werden
	//pBase = new CBase;
	//pDerived = dynamic_cast<CDerived*>(pBase);
	//if (pDerived != NULL)
	//	pDerived->ShowClass();
	//else
	//	cout << "Cast gescheitert !" << endl;

	///////////////////////////////////////////////////
	// static_cast:

	delete pDerived;
	// logischer Fehler, aber erlaubt, da pBase ein CDerived sein "knnte", keine berprfung
	pDerived = static_cast<CDerived*>(pBase);

	///////////////////////////////////////////////////
	// reinterpret_cast:

	int  nNumber = 42;
	// logischer Fehler, aber erlaubt, da beides 32-Bit-Zeiger sind
	pDerived = reinterpret_cast<CDerived*>(&nNumber);

	///////////////////////////////////////////////////
	// const_cast:

	const CDerived* pConstDerived = new CDerived;
	// Fehler, da Objekt const
	// pConstDerived->m_nNumber = 42;
	pDerived = const_cast<CDerived*>(pConstDerived);
	// pDerived darf nun manipuliert werden
	pDerived->m_nNumber = 42;

	///////////////////////////////////////////////////
	// C-Cast:

	// alles ist erlaubt
	pDerived = (CDerived*)&nNumber;
}