// /////////////////////////////////////////////////////
// Effektiv C++
// Tip 9: Copy-Konstruktor und Zuweisungsoperator fr
//        Klassen mit dynamisch alloziiertem Speicher
//
// Code: Stephan Brumme, 16.Mai 2002
//
// Zweck: - CDynamic legt Attribut m_strString dynamisch an,
//          belegt Speicher mit "Hello"
//        - fehlender Copy-Konstruktor/Zuweisungsoperator
//          berschreibt Original mit "World" und gibt es frei


// fr I/O
#include <iostream>
using namespace std;


class CDynamic
{
public:
	// Default-Konstruktor
	CDynamic()
	{
		// Speicher alloziieren
		m_strString = new char[strlen(m_strHello)+1];
		// und Zeichenkette kopieren
		strcpy(m_strString, m_strHello);
	}

	// Copy-Konstruktor
	CDynamic(const CDynamic& obj)
	{
		// Speicher alloziieren
		m_strString = new char[strlen(obj.m_strString)+1];
		// und Zeichenkette kopieren
		strcpy(m_strString, obj.m_strString);
	}

	// Zuweisungsoperator
	CDynamic& operator=(const CDynamic& obj)
	{
		// Achtung: Tip 12 fgt hier zwei wichtige Zeile hinzu !
		// if (this == &obj)
		//     return *this;

		delete[] m_strString;
		// Speicher alloziieren
		m_strString = new char[strlen(obj.m_strString)+1];
		// und Zeichenkette kopieren
		strcpy(m_strString, obj.m_strString);

		return *this;
	}

	// Destruktor
	~CDynamic()
	{
		// alte Zeichenkette berschreiben
		strcpy(m_strString, m_strWorld);
		// Speicher freigeben 
		// Absturz, wenn Copy-Konstruktor oder Zuweisungsoperator fehlt !
		delete[] m_strString;
	}

//public:	// "public" auskommentiert, um Attribut fr Debugging-Zwecke anzeigen zu knnen
	char* m_strString;


	// Konstanten deklarieren
	static const char* m_strHello;
	static const char* m_strWorld;
};


// Anfangswert fr Konstruktor
const char* CDynamic::m_strHello = "Hello";
// im Destruktor verwenden (kurz vor Freigabe)
const char* CDynamic::m_strWorld = "World";



void main()
{
	CDynamic a;
	// "Hello"
	cout << a.m_strString << endl;

	// neuer Sichtbarkeitsbereich
	{
		// Test fr Copy-Konstruktor
		CDynamic b = a;
		// "Hello"
		cout << b.m_strString << endl;

		// Test fr Zuweisungsoperator
		CDynamic c;
		c = a;
		// "Hello"
		cout << c.m_strString << endl;
	}

	// Fehler, wenn Copy-Konstruktor oder Zuweisungsoperator fehlt
	cout << a.m_strString << endl;
}